@@ -1,47 +1,49 @@
-/***************************************************************************  
- * Copyright (C) 1995-2002 MySQL AB, www.mysql.com                         *
- *                                                                         *
- * This program is free software; you can redistribute it and/or modify    *
- * it under the terms of the GNU General Public License as published by    *
- * the Free Software Foundation; either version 2 of the License, or       *
- * (at your option) any later version.                                     *
- *                                                                         *
- * This program is distributed in the hope that it will be useful,         *
- * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
- * GNU General Public License for more details.                            *
- *                                                                         *
- * You should have received a copy of the GNU General Public License       *
- * along with this program; if not, write to the Free Software Foundation  *
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA           *
-****************************************************************************/
+/* Copyright (C) 1995-2004 MySQL AB
 
-/*************************************************************************** 
- * OPTIONS.C                                                               *
- *                                                                         *
- * @description: For handling attribute APIs                               *
- *                                                                         *
- * @author     : MySQL AB(monty@mysql.com, venu@mysql.com)                 *
- * @date       : 2001-Aug-15                                               *
- * @product    : myodbc3                                                   *
- *                                                                         *
-****************************************************************************/
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
 
-/*************************************************************************** 
- * The following ODBC APIs are implemented in this file:                   *
- *                                                                         *
- *   SQLSetEnvAttr       (ISO 92)                                          *
- *   SQLGetEnvAttr       (ISO 92)                                          *
- *   SQLSetConnectAttr   (ISO 92)                                          *
- *   SQLGetConnecyAttr   (ISO 92)                                          *
- *   SQLSetStmtAttr      (ISO 92)                                          *
- *   SQLGetStmtAttr      (ISO 92)                                          *
- *   SQLSetConnectOption (ODBC, Deprecated)                                *
- *   SQLGetConnectOption (ODBC, Deprecated)                                *
- *   SQLSetStmtOption    (ODBC, Deprecated)                                *
- *   SQLGetStmtOption    (ODBC, Deprecated)                                *
- *                                                                         *
-****************************************************************************/
+   There are special exceptions to the terms and conditions of the GPL as it
+   is applied to this software. View the full text of the exception in file
+   EXCEPTIONS in the directory of this software distribution.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+
+/***************************************************************************
+ * OPTIONS.C								   *
+ *									   *
+ * @description: For handling attribute APIs				   *
+ *									   *
+ * @author     : MySQL AB(monty@mysql.com, venu@mysql.com)		   *
+ * @date       : 2001-Aug-15						   *
+ * @product    : myodbc3						   *
+ *									   *
+ ****************************************************************************/
+
+/***************************************************************************
+ * The following ODBC APIs are implemented in this file:		   *
+ *									   *
+ *   SQLSetEnvAttr	 (ISO 92)					   *
+ *   SQLGetEnvAttr	 (ISO 92)					   *
+ *   SQLSetConnectAttr	 (ISO 92)					   *
+ *   SQLGetConnecyAttr	 (ISO 92)					   *
+ *   SQLSetStmtAttr	 (ISO 92)					   *
+ *   SQLGetStmtAttr	 (ISO 92)					   *
+ *   SQLSetConnectOption (ODBC, Deprecated)				   *
+ *   SQLGetConnectOption (ODBC, Deprecated)				   *
+ *   SQLSetStmtOption	 (ODBC, Deprecated)				   *
+ *   SQLGetStmtOption	 (ODBC, Deprecated)				   *
+ *									   *
+ ****************************************************************************/
 
 #include "myodbc3.h"
 
@@ -58,16 +60,16 @@ static SQLRETURN set_constmt_attr(SQLSMALLINT  HandleType,
 {
   DBUG_ENTER("set_constmt_attr");
 
-  switch(Attribute){
+  switch(Attribute) {
 
   case SQL_ATTR_ASYNC_ENABLE:
-    if( (SQLUINTEGER)ValuePtr == SQL_ASYNC_ENABLE_ON )
+    if ((SQLUINTEGER)ValuePtr == SQL_ASYNC_ENABLE_ON)
       DBUG_RETURN(set_handle_error(HandleType,Handle,MYERR_01S02,
 				   "Doesn't support asynchronous, changed to default",0));
     break;
 
   case SQL_ATTR_CURSOR_SENSITIVITY:
-    if ((SQLUINTEGER)ValuePtr != SQL_UNSPECIFIED )
+    if ((SQLUINTEGER)ValuePtr != SQL_UNSPECIFIED)
     {
       DBUG_RETURN(set_handle_error(HandleType,Handle,MYERR_01S02,
 				   "Option value changed to default cursor sensitivity(unspecified)",0));
@@ -76,11 +78,11 @@ static SQLRETURN set_constmt_attr(SQLSMALLINT  HandleType,
 
   case SQL_ATTR_CURSOR_TYPE:
     if (((STMT FAR*)Handle)->dbc->flag & FLAG_FORWARD_CURSOR)
-    {		
-      options->cursor_type=SQL_CURSOR_FORWARD_ONLY;
+    {
+      options->cursor_type= SQL_CURSOR_FORWARD_ONLY;
       if ((SQLUINTEGER) ValuePtr != SQL_CURSOR_FORWARD_ONLY)
-        DBUG_RETURN(set_handle_error(HandleType,Handle,MYERR_01S02,
-	        		     "Forcing the use of forward-only cursor)",0));
+	DBUG_RETURN(set_handle_error(HandleType,Handle,MYERR_01S02,
+				     "Forcing the use of forward-only cursor)",0));
     }
     else if (((STMT FAR*)Handle)->dbc->flag & FLAG_DYNAMIC_CURSOR)
     {
@@ -89,7 +91,7 @@ static SQLRETURN set_constmt_attr(SQLSMALLINT  HandleType,
 
       else
       {
-	options->cursor_type=SQL_CURSOR_STATIC;
+	options->cursor_type= SQL_CURSOR_STATIC;
 	DBUG_RETURN(set_handle_error(HandleType,Handle,MYERR_01S02,
 				     "Option value changed to default static cursor",0));
       }
@@ -102,7 +104,7 @@ static SQLRETURN set_constmt_attr(SQLSMALLINT  HandleType,
 
       else
       {
-	options->cursor_type=SQL_CURSOR_STATIC;
+	options->cursor_type= SQL_CURSOR_STATIC;
 	DBUG_RETURN(set_handle_error(HandleType,Handle,MYERR_01S02,
 				     "Option value changed to default static cursor",0));
       }
@@ -110,11 +112,11 @@ static SQLRETURN set_constmt_attr(SQLSMALLINT  HandleType,
     break;
 
   case SQL_ATTR_MAX_LENGTH:
-    options->max_length = (SQLUINTEGER) ValuePtr;
+    options->max_length= (SQLUINTEGER) ValuePtr;
     break;
 
   case SQL_ATTR_MAX_ROWS:
-    options->max_rows = (SQLUINTEGER) ValuePtr;
+    options->max_rows= (SQLUINTEGER) ValuePtr;
     break;
 
   case SQL_ATTR_METADATA_ID:
@@ -128,17 +130,17 @@ static SQLRETURN set_constmt_attr(SQLSMALLINT  HandleType,
     break;
 
   case SQL_ATTR_ROW_BIND_TYPE:
-    options->bind_type = (SQLUINTEGER )ValuePtr;
+    options->bind_type= (SQLUINTEGER) ValuePtr;
     break;
 
   case SQL_ATTR_SIMULATE_CURSOR:
-    if ((SQLUINTEGER)ValuePtr != SQL_SC_TRY_UNIQUE )
+    if ((SQLUINTEGER)ValuePtr != SQL_SC_TRY_UNIQUE)
       DBUG_RETURN(set_handle_error(HandleType,Handle,MYERR_01S02,
 				   "Option value changed to default cursor simulation",0));
     break;
 
   case SQL_ATTR_ROW_BIND_OFFSET_PTR:
-    options->bind_offset = (SQLINTEGER *)ValuePtr;
+    options->bind_offset= (SQLINTEGER *)ValuePtr;
     break;
 
   case 1226:/* MS SQL Server Extension */
@@ -166,17 +168,19 @@ static SQLRETURN set_constmt_attr(SQLSMALLINT  HandleType,
   DBUG_RETURN_STATUS(SQL_SUCCESS);
 }
 
+
 /*
   @type    : myodbc3 internal
   @purpose : returns the common connection/stmt attributes
 */
 
-static SQLRETURN get_constmt_attr(SQLSMALLINT  HandleType,
-				  SQLHANDLE    Handle,
-				  STMT_OPTIONS *options,
-				  SQLINTEGER   Attribute,
-				  SQLPOINTER   ValuePtr,
-				  SQLINTEGER   *StringLengthPtr)
+static SQLRETURN
+get_constmt_attr(SQLSMALLINT  HandleType,
+		 SQLHANDLE    Handle,
+		 STMT_OPTIONS *options,
+		 SQLINTEGER   Attribute,
+		 SQLPOINTER   ValuePtr,
+		 SQLINTEGER   *StringLengthPtr __attribute__((unused)))
 {
   DBUG_ENTER("get_constmt_attr");
 
@@ -227,19 +231,19 @@ static SQLRETURN get_constmt_attr(SQLSMALLINT  HandleType,
     break;
 
   case SQL_KEYSET_SIZE:
-    *((SQLUINTEGER *) ValuePtr)=0L;
+    *((SQLUINTEGER *) ValuePtr)= 0L;
     break;
 
   case SQL_ROWSET_SIZE:
-    *(SQLUINTEGER *)ValuePtr = options->rows_in_set;
+    *(SQLUINTEGER *)ValuePtr= options->rows_in_set;
     break;
 
   case SQL_NOSCAN:
-    *((SQLUINTEGER *) ValuePtr)=SQL_NOSCAN_ON;
+    *((SQLUINTEGER *) ValuePtr)= SQL_NOSCAN_ON;
     break;
 
   case SQL_ATTR_ROW_BIND_OFFSET_PTR:
-    *((SQLINTEGER *) ValuePtr) = options->bind_offset ?
+    *((SQLINTEGER *) ValuePtr)= options->bind_offset ?
       *(options->bind_offset):
       0;
     break;
@@ -263,17 +267,19 @@ static SQLRETURN get_constmt_attr(SQLSMALLINT  HandleType,
   DBUG_RETURN_STATUS(SQL_SUCCESS);
 }
 
+
 /*
   @type    : myodbc3 internal
   @purpose : sets the connection attributes
 */
 
-static SQLRETURN set_con_attr(SQLHDBC    hdbc,
-			      SQLINTEGER Attribute,
-			      SQLPOINTER ValuePtr,
-			      SQLINTEGER StringLengthPtr)
+static SQLRETURN
+set_con_attr(SQLHDBC	hdbc,
+	     SQLINTEGER Attribute,
+	     SQLPOINTER ValuePtr,
+	     SQLINTEGER StringLengthPtr)
 {
-  DBC FAR *dbc=(DBC FAR*) hdbc;
+  DBC FAR *dbc= (DBC FAR*) hdbc;
   DBUG_ENTER("SQLSetConnectAttr");
   DBUG_PRINT("enter",("Atrr: %d, ValuePtr: 0x%lx, StrLenPtr: %d",Attribute,ValuePtr,StringLengthPtr));
 
@@ -289,58 +295,64 @@ static SQLRETURN set_con_attr(SQLHDBC    hdbc,
     {
       if (!dbc->server) /* no connection yet */
       {
-        dbc->commit_flag = CHECK_AUTOCOMMIT_OFF;        
-        DBUG_RETURN_STATUS(SQL_SUCCESS);
+	dbc->commit_flag= CHECK_AUTOCOMMIT_OFF;
+	DBUG_RETURN_STATUS(SQL_SUCCESS);
       }
       if (!(trans_supported(dbc)) || (dbc->flag & FLAG_NO_TRANSACTIONS))
-        DBUG_RETURN(set_conn_error(dbc,MYERR_S1C00,
-           "Transactions are not enabled", 4000));
+	DBUG_RETURN(set_conn_error(dbc,MYERR_S1C00,
+				   "Transactions are not enabled", 4000));
 
       if (autocommit_on(dbc))
-        DBUG_RETURN(odbc_stmt(dbc,"SET AUTOCOMMIT=0"));
+	DBUG_RETURN(odbc_stmt(dbc,"SET AUTOCOMMIT=0"));
     }
     else if (!dbc->server) /* no connection yet */
     {
-      dbc->commit_flag = CHECK_AUTOCOMMIT_ON;        
+      dbc->commit_flag= CHECK_AUTOCOMMIT_ON;
       DBUG_RETURN_STATUS(SQL_SUCCESS);
     }
     else if (trans_supported(dbc) && !(autocommit_on(dbc)))
       DBUG_RETURN(odbc_stmt(dbc,"SET AUTOCOMMIT=1"));
-
     break;
 
   case SQL_ATTR_CONNECTION_TIMEOUT:
-    {
-      uint timeout = (SQLUINTEGER)ValuePtr;
-      DBUG_RETURN(mysql_options(&dbc->mysql, MYSQL_OPT_CONNECT_TIMEOUT,
-                                (const char *)&timeout));
-    }
-    break;
+  {
+    uint timeout= (SQLUINTEGER)ValuePtr;
+    DBUG_RETURN(mysql_options(&dbc->mysql, MYSQL_OPT_CONNECT_TIMEOUT,
+			      (const char *)&timeout));
+  }
+  break;
 
   case SQL_ATTR_CURRENT_CATALOG:
+  {
+    char ldb[NAME_LEN+1], *db;
+    
+    if (!(db= fix_str((char *)ldb, (char *)ValuePtr, StringLengthPtr)))
+      DBUG_RETURN(set_conn_error(hdbc,MYERR_S1009,NULL, 0));
+
     pthread_mutex_lock(&dbc->lock);
-    if (mysql_select_db(&dbc->mysql,(char*) ValuePtr))
+    if (mysql_select_db(&dbc->mysql,(char*) db))
     {
       set_conn_error(dbc,MYERR_S1000,mysql_error(&dbc->mysql),
-           mysql_errno(&dbc->mysql));
+		     mysql_errno(&dbc->mysql));
       pthread_mutex_unlock(&dbc->lock);
       DBUG_RETURN_STATUS(SQL_ERROR);
     }
     my_free((gptr) dbc->database,MYF(0));
-    dbc->database=my_strdup((char*) ValuePtr,MYF(MY_WME));
+    dbc->database= my_strdup(db,MYF(MY_WME));
     pthread_mutex_unlock(&dbc->lock);
-    break;
+  }
+  break;
 
   case SQL_ATTR_LOGIN_TIMEOUT:
-    dbc->login_timeout=(SQLUINTEGER)ValuePtr;
+    dbc->login_timeout= (SQLUINTEGER)ValuePtr;
     break;
 
   case SQL_ATTR_ODBC_CURSORS:
-    if ((dbc->flag & FLAG_FORWARD_CURSOR) && 
-        (SQLUINTEGER)ValuePtr != SQL_CUR_USE_ODBC)
+    if ((dbc->flag & FLAG_FORWARD_CURSOR) &&
+	(SQLUINTEGER)ValuePtr != SQL_CUR_USE_ODBC)
       DBUG_RETURN(set_conn_error(hdbc,MYERR_01S02,
-		  "Forcing the Driver Manager to use ODBC cursor library",0));
-    break;	  
+				 "Forcing the Driver Manager to use ODBC cursor library",0));
+    break;
 
   case SQL_OPT_TRACE:
   case SQL_OPT_TRACEFILE:
@@ -349,7 +361,7 @@ static SQLRETURN set_con_attr(SQLHDBC    hdbc,
   case SQL_TRANSLATE_OPTION:
   {
     char buff[100];
-    sprintf(buff,"Suppose to set this attribute '%d' through driver manager, not by the driver",Attribute);
+    sprintf(buff,"Suppose to set this attribute '%d' through driver manager, not by the driver",(int) Attribute);
     DBUG_RETURN(set_conn_error(hdbc,MYERR_01S02,buff,0));
   }
 
@@ -361,7 +373,7 @@ static SQLRETURN set_con_attr(SQLHDBC    hdbc,
   case SQL_ATTR_TXN_ISOLATION:
     if (!dbc->server)  /* no connection yet */
     {
-      dbc->txn_isolation = (SQLINTEGER)ValuePtr;        
+      dbc->txn_isolation= (SQLINTEGER)ValuePtr;
       DBUG_RETURN_STATUS(SQL_SUCCESS);
     }
     if (trans_supported(dbc))
@@ -379,16 +391,17 @@ static SQLRETURN set_con_attr(SQLHDBC    hdbc,
 	level="READ UNCOMMITTED";
       sprintf(buff,"SET SESSION TRANSACTION ISOLATION LEVEL %s",level);
       if (odbc_stmt(dbc,buff) == SQL_SUCCESS)
-	dbc->txn_isolation=(SQLINTEGER)ValuePtr;
+	dbc->txn_isolation= (SQLINTEGER)ValuePtr;
     }
     else
       DBUG_PRINT("info",("SQL_ATTR_TXN_ISOLATION %ld ignored",
 			 (SQLINTEGER)ValuePtr));
     break;
 
-    /* 3.x driver doesn't support any statement attributes
-       at connection level, but to make sure all 2.x apps
-       works fine...lets support it..nothing to loose..
+    /*
+      3.x driver doesn't support any statement attributes
+      at connection level, but to make sure all 2.x apps
+      works fine...lets support it..nothing to loose..
     */
   default:
     DBUG_RETURN(set_constmt_attr(2,dbc,&dbc->stmt_options,
@@ -397,32 +410,34 @@ static SQLRETURN set_con_attr(SQLHDBC    hdbc,
   DBUG_RETURN_STATUS(SQL_SUCCESS);
 }
 
+
 /*
   @type    : myodbc3 internal
   @purpose : returns the connection attribute values
 */
 
-static SQLRETURN get_con_attr(SQLHDBC    hdbc,
+static SQLRETURN get_con_attr(SQLHDBC	 hdbc,
 			      SQLINTEGER Attribute,
 			      SQLPOINTER ValuePtr,
 			      SQLINTEGER BufferLength,
 			      SQLINTEGER *StringLengthPtr)
 {
-  DBC FAR *dbc=(DBC FAR*) hdbc;
-  SQLRETURN result=SQL_SUCCESS;
+  DBC FAR *dbc= (DBC FAR*) hdbc;
+  SQLRETURN result= SQL_SUCCESS;
   SQLINTEGER strlen;
-  SQLPOINTER vparam=0;
+  SQLPOINTER vparam= 0;
   DBUG_ENTER("SQLGetConnectAttr");
-  DBUG_PRINT("enter",("Atrr: %d, ValuePtr: 0x%lx, BufLen: %d, StrLenPtr: 0x%lx",Attribute,ValuePtr,BufferLength,
-		      StringLengthPtr));
+  DBUG_PRINT("enter",
+	     ("Atrr: %d, ValuePtr: 0x%lx, BufLen: %d, StrLenPtr: 0x%lx",
+	      Attribute,ValuePtr,BufferLength, StringLengthPtr));
 
   if (!ValuePtr)
-    ValuePtr = vparam;
+    ValuePtr= vparam;
 
   if (!StringLengthPtr)
-    StringLengthPtr = &strlen;
+    StringLengthPtr= &strlen;
 
-  switch(Attribute){
+  switch(Attribute) {
 
   case SQL_ATTR_ACCESS_MODE:
     *((SQLUINTEGER *) ValuePtr)= SQL_MODE_READ_WRITE;
@@ -433,8 +448,8 @@ static SQLRETURN get_con_attr(SQLHDBC    hdbc,
     break;
 
   case SQL_ATTR_AUTOCOMMIT:
-    *((SQLUINTEGER *)ValuePtr)= (autocommit_on(dbc) || 
-				!(trans_supported(dbc)) ?
+    *((SQLUINTEGER *)ValuePtr)= (autocommit_on(dbc) ||
+				 !(trans_supported(dbc)) ?
 				 SQL_AUTOCOMMIT_ON :
 				 SQL_AUTOCOMMIT_OFF);
     break;
@@ -458,8 +473,9 @@ static SQLRETURN get_con_attr(SQLHDBC    hdbc,
     if (reget_current_catalog(dbc))
       result= SQL_ERROR;
     else
-      *StringLengthPtr=(SQLSMALLINT) (strmake((char*)ValuePtr,dbc->database,
-                       BufferLength) - (char*) ValuePtr);
+      *StringLengthPtr= (SQLSMALLINT) (strmake((char*)ValuePtr,dbc->database,
+					       BufferLength) -
+				       (char*) ValuePtr);
     break;
 
   case SQL_ATTR_LOGIN_TIMEOUT:
@@ -467,11 +483,11 @@ static SQLRETURN get_con_attr(SQLHDBC    hdbc,
     break;
 
   case SQL_ATTR_ODBC_CURSORS:
-    if ((dbc->flag & FLAG_FORWARD_CURSOR))	  
+    if ((dbc->flag & FLAG_FORWARD_CURSOR))
       *((SQLUINTEGER *) ValuePtr)= SQL_CUR_USE_ODBC;
-    else 	  	  
+    else
       *((SQLUINTEGER *) ValuePtr)= SQL_CUR_USE_IF_NEEDED;
-    break;	  
+    break;
 
   case SQL_OPT_TRACE:
   case SQL_OPT_TRACEFILE:
@@ -480,8 +496,10 @@ static SQLRETURN get_con_attr(SQLHDBC    hdbc,
   case SQL_TRANSLATE_OPTION:
   {
     char buff[100];
-    sprintf(buff,"Suppose to get this attribute '%d' through driver manager, not by the driver",Attribute);
-    result = set_conn_error(hdbc,MYERR_01S02,buff,0);
+    sprintf(buff,
+	    "Suppose to get this attribute '%d' through driver manager, not by the driver",
+	    (int) Attribute);
+    result= set_conn_error(hdbc,MYERR_01S02,buff,0);
     break;
   }
 
@@ -499,64 +517,69 @@ static SQLRETURN get_con_attr(SQLHDBC    hdbc,
       works fine...lets support it..nothing to loose..
     */
   default:
-    result=get_constmt_attr(2,dbc,&dbc->stmt_options,
-			    Attribute,ValuePtr,
-			    StringLengthPtr);
+    result= get_constmt_attr(2,dbc,&dbc->stmt_options,
+			     Attribute,ValuePtr,
+			     StringLengthPtr);
   }
   DBUG_RETURN_STATUS(result);
 }
 
+
 /*
   @type    : myodbc3 internal
   @purpose : sets the statement attributes
 */
 
-static SQLRETURN set_stmt_attr(SQLHSTMT   hstmt,
-			       SQLINTEGER Attribute,
-			       SQLPOINTER ValuePtr,
-			       SQLINTEGER StringLengthPtr)
+static SQLRETURN
+set_stmt_attr(SQLHSTMT	 hstmt,
+	      SQLINTEGER Attribute,
+	      SQLPOINTER ValuePtr,
+	      SQLINTEGER StringLengthPtr __attribute__((unused)))
 {
-  STMT FAR *stmt=(STMT FAR*) hstmt;
-  SQLRETURN result=SQL_SUCCESS;
-  STMT_OPTIONS *options = &stmt->stmt_options;
+  STMT FAR *stmt= (STMT FAR*) hstmt;
+  SQLRETURN result= SQL_SUCCESS;
+  STMT_OPTIONS *options= &stmt->stmt_options;
   DBUG_ENTER("SQLSetStmtAttr");
-  DBUG_PRINT("enter",("Atrr: %d, ValuePtr: 0x%lx, StrLenPtr: %d",Attribute,ValuePtr,StringLengthPtr));
+  DBUG_PRINT("enter",
+	     ("Atrr: %d, ValuePtr: 0x%lx, StrLenPtr: %d",
+	      Attribute,ValuePtr,StringLengthPtr));
 
-  switch(Attribute){
+  switch (Attribute) {
 
   case SQL_ATTR_CURSOR_SCROLLABLE:
-    if ((SQLUINTEGER)ValuePtr == SQL_NONSCROLLABLE && 
-        options->cursor_type != SQL_CURSOR_FORWARD_ONLY)
-      options->cursor_type = SQL_CURSOR_FORWARD_ONLY;   
+    if ((SQLUINTEGER)ValuePtr == SQL_NONSCROLLABLE &&
+	options->cursor_type != SQL_CURSOR_FORWARD_ONLY)
+      options->cursor_type= SQL_CURSOR_FORWARD_ONLY;
 
-    else if ((SQLUINTEGER)ValuePtr == SQL_SCROLLABLE && 
+    else if ((SQLUINTEGER)ValuePtr == SQL_SCROLLABLE &&
 	     options->cursor_type == SQL_CURSOR_FORWARD_ONLY)
-      options->cursor_type = SQL_CURSOR_STATIC;
+      options->cursor_type= SQL_CURSOR_STATIC;
     break;
 
   case SQL_ATTR_AUTO_IPD:
-    if ((SQLUINTEGER) ValuePtr != SQL_FALSE )
+    if ((SQLUINTEGER) ValuePtr != SQL_FALSE)
       DBUG_RETURN(set_error(hstmt,MYERR_01S02,
 			    "Option value changed to default auto ipd",0));
     break;
 
   case SQL_ATTR_PARAM_STATUS_PTR: /* need to support this ....*/
-    options->paramStatusPtr = (SQLUSMALLINT *)ValuePtr;
+    options->paramStatusPtr= (SQLUSMALLINT *)ValuePtr;
     break;
 
   case SQL_ATTR_PARAMS_PROCESSED_PTR: /* need to support this ....*/
-    options->paramProcessedPtr = (SQLUINTEGER *)ValuePtr;
+    options->paramProcessedPtr= (SQLUINTEGER *)ValuePtr;
     break;
 
   case SQL_ATTR_PARAMSET_SIZE:
-    if ((SQLUINTEGER)ValuePtr != 1 )
+    if ((SQLUINTEGER)ValuePtr != 1)
       DBUG_RETURN(set_error(hstmt,MYERR_01S02,
-			    "Option value changed to default parameter size",0));
+			    "Option value changed to default parameter size",
+			    0));
     break;
 
   case SQL_ATTR_ROW_ARRAY_SIZE:
   case SQL_ROWSET_SIZE:
-    options->rows_in_set = (SQLUINTEGER)ValuePtr;
+    options->rows_in_set= (SQLUINTEGER)ValuePtr;
     break;
 
   case SQL_ATTR_ROW_NUMBER:
@@ -565,33 +588,34 @@ static SQLRETURN set_stmt_attr(SQLHSTMT   hstmt,
     break;
 
   case SQL_ATTR_ROW_OPERATION_PTR:
-    options->rowOperationPtr = (SQLUSMALLINT *)ValuePtr;
+    options->rowOperationPtr= (SQLUSMALLINT *)ValuePtr;
     break;
 
   case SQL_ATTR_ROW_STATUS_PTR:
-    options->rowStatusPtr = (SQLUSMALLINT *)ValuePtr;
+    options->rowStatusPtr= (SQLUSMALLINT *)ValuePtr;
     break;
 
   case SQL_ATTR_ROWS_FETCHED_PTR:
-    options->rowsFetchedPtr = (SQLUINTEGER *)ValuePtr;
+    options->rowsFetchedPtr= (SQLUINTEGER *)ValuePtr;
     break;
 
   case SQL_ATTR_SIMULATE_CURSOR:
-    options->simulateCursor = (SQLUINTEGER)ValuePtr;
+    options->simulateCursor= (SQLUINTEGER)ValuePtr;
     break;
 
     /*
       3.x driver doesn't support any statement attributes
-       at connection level, but to make sure all 2.x apps
-       works fine...lets support it..nothing to loose..
+      at connection level, but to make sure all 2.x apps
+      works fine...lets support it..nothing to loose..
     */
   default:
-    result = set_constmt_attr(3,hstmt,options,
-			      Attribute,ValuePtr);
+    result= set_constmt_attr(3,hstmt,options,
+			     Attribute,ValuePtr);
   }
   DBUG_RETURN_STATUS(result);
 }
 
+
 /*
   @type    : myodbc3 internal
   @purpose : returns the statement attribute values
@@ -600,71 +624,72 @@ static SQLRETURN set_stmt_attr(SQLHSTMT   hstmt,
 static SQLRETURN get_stmt_attr(SQLHSTMT   hstmt,
 			       SQLINTEGER Attribute,
 			       SQLPOINTER ValuePtr,
-			       SQLINTEGER BufferLength,
+			       SQLINTEGER BufferLength __attribute__((unused)),
 			       SQLINTEGER *StringLengthPtr)
 {
-  SQLRETURN result=SQL_SUCCESS;
-  STMT FAR *stmt=(STMT FAR*) hstmt;
-  STMT_OPTIONS *options = &stmt->stmt_options;
+  SQLRETURN result= SQL_SUCCESS;
+  STMT FAR *stmt= (STMT FAR*) hstmt;
+  STMT_OPTIONS *options= &stmt->stmt_options;
   SQLPOINTER vparam;
   SQLINTEGER strlen;
   DBUG_ENTER("SQLGetStmtAttr");
-  DBUG_PRINT("enter",("Atrr: %d, ValuePtr: 0x%lx, BufLen: %d, StrLenPtr: 0x%lx",
-              Attribute,ValuePtr,BufferLength,StringLengthPtr));
+  DBUG_PRINT("enter",
+	     ("Atrr: %d, ValuePtr: 0x%lx, BufLen: %d, StrLenPtr: 0x%lx",
+	      Attribute,ValuePtr,BufferLength,StringLengthPtr));
 
   if (!ValuePtr)
-    ValuePtr = &vparam;
+    ValuePtr= &vparam;
 
   if (!StringLengthPtr)
-    StringLengthPtr = &strlen;
+    StringLengthPtr= &strlen;
 
   switch(Attribute) {
 
   case SQL_ATTR_CURSOR_SCROLLABLE:
     if (options->cursor_type == SQL_CURSOR_FORWARD_ONLY)
-      *(SQLUINTEGER*)ValuePtr = SQL_NONSCROLLABLE;
+      *(SQLUINTEGER*)ValuePtr= SQL_NONSCROLLABLE;
     else
-      *(SQLUINTEGER*)ValuePtr = SQL_SCROLLABLE;
+      *(SQLUINTEGER*)ValuePtr= SQL_SCROLLABLE;
     break;
 
   case SQL_ATTR_AUTO_IPD:
-    *(SQLUINTEGER *)ValuePtr = SQL_FALSE;
+    *(SQLUINTEGER *)ValuePtr= SQL_FALSE;
     break;
 
   case SQL_ATTR_PARAM_STATUS_PTR: /* need to support this ....*/
-    ValuePtr = (SQLUSMALLINT *)options->paramStatusPtr;
+    ValuePtr= (SQLUSMALLINT *)options->paramStatusPtr;
     break;
 
   case SQL_ATTR_PARAMS_PROCESSED_PTR: /* need to support this ....*/
-    ValuePtr = (SQLUSMALLINT *)options->paramProcessedPtr;
+    ValuePtr= (SQLUSMALLINT *)options->paramProcessedPtr;
     break;
 
   case SQL_ATTR_PARAMSET_SIZE:
-    *(SQLUINTEGER *)ValuePtr = 1;
+    *(SQLUINTEGER *)ValuePtr= 1;
     break;
 
   case SQL_ATTR_ROW_ARRAY_SIZE:
-    *(SQLUINTEGER *)ValuePtr = options->rows_in_set;
+    *(SQLUINTEGER *)ValuePtr= options->rows_in_set;
     break;
 
   case SQL_ATTR_ROW_NUMBER:
-    *(SQLUINTEGER *)ValuePtr = stmt->current_row+1;
+    *(SQLUINTEGER *)ValuePtr= stmt->current_row+1;
     break;
 
   case SQL_ATTR_ROW_OPERATION_PTR: /* need to support this ....*/
-    ValuePtr = (SQLUSMALLINT *)options->rowOperationPtr;
+    ValuePtr= (SQLUSMALLINT *)options->rowOperationPtr;
     break;
 
   case SQL_ATTR_ROW_STATUS_PTR:
-    ValuePtr = (SQLUSMALLINT *)options->rowStatusPtr;
+    ValuePtr= (SQLUSMALLINT *)options->rowStatusPtr;
     break;
 
   case SQL_ATTR_ROWS_FETCHED_PTR:
-    ValuePtr = (SQLUINTEGER *)options->rowsFetchedPtr;
+    ValuePtr= (SQLUINTEGER *)options->rowsFetchedPtr;
     break;
 
   case SQL_ATTR_SIMULATE_CURSOR:
-    ValuePtr = (SQLUSMALLINT *)options->simulateCursor;
+    ValuePtr= (SQLUSMALLINT *)options->simulateCursor;
     break;
 
     /*
@@ -674,39 +699,40 @@ static SQLRETURN get_stmt_attr(SQLHSTMT   hstmt,
       - MS ODB DM is crashing at the time of stmt allocation
     */
   case SQL_ATTR_APP_ROW_DESC:
-    *(SQLPOINTER *)ValuePtr = &stmt->ard;
-    *StringLengthPtr = sizeof(SQL_IS_POINTER);
+    *(SQLPOINTER *)ValuePtr= &stmt->ard;
+    *StringLengthPtr= sizeof(SQL_IS_POINTER);
     break;
 
   case SQL_ATTR_IMP_ROW_DESC:
-    *(SQLPOINTER *)ValuePtr = &stmt->ird;
-    *StringLengthPtr = sizeof(SQL_IS_POINTER);
+    *(SQLPOINTER *)ValuePtr= &stmt->ird;
+    *StringLengthPtr= sizeof(SQL_IS_POINTER);
     break;
 
   case SQL_ATTR_APP_PARAM_DESC:
-    *(SQLPOINTER *)ValuePtr = &stmt->apd;
-    *StringLengthPtr = sizeof(SQL_IS_POINTER);
+    *(SQLPOINTER *)ValuePtr= &stmt->apd;
+    *StringLengthPtr= sizeof(SQL_IS_POINTER);
     break;
 
   case SQL_ATTR_IMP_PARAM_DESC:
-    *(SQLPOINTER *)ValuePtr = &stmt->ipd;
-    *StringLengthPtr = sizeof(SQL_IS_POINTER);
+    *(SQLPOINTER *)ValuePtr= &stmt->ipd;
+    *StringLengthPtr= sizeof(SQL_IS_POINTER);
     break;
 
     /*
       3.x driver doesn't support any statement attributes
-       at connection level, but to make sure all 2.x apps
-       works fine...lets support it..nothing to loose..
+      at connection level, but to make sure all 2.x apps
+      works fine...lets support it..nothing to loose..
     */
   default:
-    result=get_constmt_attr(3,hstmt,options,
-			    Attribute,ValuePtr,
-			    StringLengthPtr);
+    result= get_constmt_attr(3,hstmt,options,
+			     Attribute,ValuePtr,
+			     StringLengthPtr);
   }
 
   DBUG_RETURN_STATUS(result);
 }
 
+
 /*
   @type    : ODBC 1.0 API
   @purpose : sets the connection options
@@ -715,13 +741,14 @@ static SQLRETURN get_stmt_attr(SQLHSTMT   hstmt,
 SQLRETURN SQL_API SQLSetConnectOption(SQLHDBC hdbc, SQLUSMALLINT fOption,
 				      SQLUINTEGER  vParam)
 {
-  SQLRETURN result=SQL_SUCCESS;
+  SQLRETURN result= SQL_SUCCESS;
   DBUG_ENTER("SQLSetConnectOption");
 
-  result=set_con_attr(hdbc,fOption,(SQLPOINTER)vParam,SQL_NTS);
+  result= set_con_attr(hdbc,fOption,(SQLPOINTER)vParam,SQL_NTS);
   DBUG_RETURN_STATUS(result);
 }
 
+
 /*
   @type    : ODBC 1.0 API
   @purpose : returns the connection options
@@ -730,13 +757,14 @@ SQLRETURN SQL_API SQLSetConnectOption(SQLHDBC hdbc, SQLUSMALLINT fOption,
 SQLRETURN SQL_API SQLGetConnectOption(SQLHDBC hdbc,SQLUSMALLINT fOption,
 				      SQLPOINTER vParam)
 {
-  SQLRETURN result=SQL_SUCCESS;
+  SQLRETURN result= SQL_SUCCESS;
   DBUG_ENTER("SQLGetConnectOption");
 
-  result=get_con_attr(hdbc,fOption,vParam,SQL_NTS,(SQLINTEGER *)NULL);
+  result= get_con_attr(hdbc,fOption,vParam,SQL_NTS,(SQLINTEGER *)NULL);
   DBUG_RETURN_STATUS(result);
 }
 
+
 /*
   @type    : ODBC 1.0 API
   @purpose : sets the statement options
@@ -745,13 +773,14 @@ SQLRETURN SQL_API SQLGetConnectOption(SQLHDBC hdbc,SQLUSMALLINT fOption,
 SQLRETURN SQL_API SQLSetStmtOption(SQLHSTMT hstmt,SQLUSMALLINT fOption,
 				   SQLUINTEGER vParam)
 {
-  SQLRETURN result=SQL_SUCCESS;
+  SQLRETURN result= SQL_SUCCESS;
   DBUG_ENTER("SQLSetStmtOption");
 
-  result=set_stmt_attr(hstmt,fOption,(SQLPOINTER)vParam,SQL_NTS);
+  result= set_stmt_attr(hstmt,fOption,(SQLPOINTER)vParam,SQL_NTS);
   DBUG_RETURN_STATUS(result);
 }
 
+
 /*
   @type    : ODBC 1.0 API
   @purpose : returns the statement options
@@ -760,22 +789,24 @@ SQLRETURN SQL_API SQLSetStmtOption(SQLHSTMT hstmt,SQLUSMALLINT fOption,
 SQLRETURN SQL_API SQLGetStmtOption(SQLHSTMT hstmt,SQLUSMALLINT fOption,
 				   SQLPOINTER vParam)
 {
-  SQLRETURN result=SQL_SUCCESS;
+  SQLRETURN result= SQL_SUCCESS;
   DBUG_ENTER("SQLGetStmtOption");
 
-  result=get_stmt_attr(hstmt,fOption,vParam,SQL_NTS,(SQLINTEGER *)NULL);
+  result= get_stmt_attr(hstmt,fOption,vParam,SQL_NTS,(SQLINTEGER *)NULL);
   DBUG_RETURN_STATUS(result);
 }
 
+
 /*
   @type    : ODBC 3.0 API
   @purpose : sets the environment attributes
 */
 
-SQLRETURN SQL_API SQLSetEnvAttr(SQLHENV    henv,
-				SQLINTEGER Attribute,
-				SQLPOINTER ValuePtr,
-				SQLINTEGER StringLength)
+SQLRETURN SQL_API
+SQLSetEnvAttr(SQLHENV	 henv,
+	      SQLINTEGER Attribute,
+	      SQLPOINTER ValuePtr,
+	      SQLINTEGER StringLength __attribute__((unused)))
 {
   DBUG_ENTER("SQLSetEnvAttr");
   DBUG_PRINT("enter",("Atrr: %d, ValuePtr: 0x%lx",Attribute,ValuePtr));
@@ -786,11 +817,11 @@ SQLRETURN SQL_API SQLSetEnvAttr(SQLHENV    henv,
   switch (Attribute) {
 
   case SQL_ATTR_ODBC_VERSION:
-    ((ENV FAR *)henv)->odbc_ver = (SQLINTEGER)ValuePtr;
+    ((ENV FAR *)henv)->odbc_ver= (SQLINTEGER)ValuePtr;
     break;
 
   case SQL_ATTR_OUTPUT_NTS:
-    if ((SQLINTEGER)ValuePtr == SQL_TRUE )
+    if ((SQLINTEGER)ValuePtr == SQL_TRUE)
       break;
 
   default:
@@ -799,16 +830,18 @@ SQLRETURN SQL_API SQLSetEnvAttr(SQLHENV    henv,
   DBUG_RETURN_STATUS(SQL_SUCCESS);
 }
 
+
 /*
   @type    : ODBC 3.0 API
   @purpose : returns the environment attributes
 */
 
-SQLRETURN SQL_API SQLGetEnvAttr(SQLHENV    henv,
-				SQLINTEGER Attribute,
-				SQLPOINTER ValuePtr,
-				SQLINTEGER BufferLength,
-				SQLINTEGER *StringLengthPtr)
+SQLRETURN SQL_API
+SQLGetEnvAttr(SQLHENV	 henv,
+	      SQLINTEGER Attribute,
+	      SQLPOINTER ValuePtr,
+	      SQLINTEGER BufferLength __attribute__((unused)),
+	      SQLINTEGER *StringLengthPtr __attribute__((unused)))
 {
   DBUG_ENTER("SQLGetEnvAttr");
   DBUG_PRINT("enter",("Atrr: %d",Attribute));
@@ -816,11 +849,11 @@ SQLRETURN SQL_API SQLGetEnvAttr(SQLHENV    henv,
   switch(Attribute) {
 
   case SQL_ATTR_ODBC_VERSION:
-    *(SQLINTEGER*)ValuePtr = ((ENV FAR *)henv)->odbc_ver;
+    *(SQLINTEGER*)ValuePtr= ((ENV FAR *)henv)->odbc_ver;
     break;
 
   case SQL_ATTR_OUTPUT_NTS:
-    *((SQLINTEGER*)ValuePtr) = SQL_TRUE;
+    *((SQLINTEGER*)ValuePtr)= SQL_TRUE;
     break;
 
   default:
@@ -829,6 +862,7 @@ SQLRETURN SQL_API SQLGetEnvAttr(SQLHENV    henv,
   DBUG_RETURN_STATUS(SQL_SUCCESS);
 }
 
+
 /*
   @type    : ODBC 3.0 API
   @purpose : sets the connection attributes
@@ -839,8 +873,7 @@ SQLRETURN SQL_API SQLSetConnectAttr(SQLHDBC hdbc,
 				    SQLPOINTER ValuePtr,
 				    SQLINTEGER StringLength)
 {
-  return set_con_attr(hdbc,Attribute,
-		      ValuePtr,StringLength);
+  return set_con_attr(hdbc,Attribute, ValuePtr,StringLength);
 }
 
 /*
@@ -854,8 +887,7 @@ SQLRETURN SQL_API SQLGetConnectAttr(SQLHDBC hdbc,
 				    SQLINTEGER BufferLength,
 				    SQLINTEGER *StringLengthPtr)
 {
-  return get_con_attr(hdbc,Attribute,
-		      ValuePtr,BufferLength,
+  return get_con_attr(hdbc,Attribute, ValuePtr,BufferLength,
 		      StringLengthPtr);
 }
 
